==> org-lsst-ccs-subsystem-fcs-main-2.1.4-sources.jar: org/lsst/ccs/subsystems/fcs/singlefiltertest/SftFilterLatchModule.java <==
/*
 * To change this template, choose Tools | Templates
 * and on the template in the editor.
 */
package org.lsst.ccs.subsystems.fcs.singlefiltertest;

import java.util.concurrent.locks.Condition;
import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.messaging.BadCommandException;
import org.lsst.ccs.messaging.ErrorInCommandExecutionException;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.framework.annotations.ConfigChanger;
import org.lsst.ccs.subsystems.fcs.CarouselModule;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.FilterPresenceInLatchStatus;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.LockStatus;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.MobileItemAction;
import org.lsst.ccs.subsystems.fcs.MainModule;
import org.lsst.ccs.subsystems.fcs.NumericSensor;
import org.lsst.ccs.subsystems.fcs.StatusDataPublishedByAutochangerLatch;
import org.lsst.ccs.subsystems.fcs.common.MobileItemModule;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;

/**
 * This is a model for a latch. A latch is what holds a filter in the trucks of
 * the autochanger. There is 2 latches : one on each side of the filter. The
 * state of the latches are monitoring by digital sensors : one sensor to know
 * if a filter is in the latch or not and 2 status sensors to know if the latch
 * is EngageBrakeed or ReleaseBrakeed. If the 2 status sensors are inconsistent,
 * the latch is in error.
 * This class is used in single-filter-test.
 * For the final product the class AutochangerLatchModule is used.
 *
 * @author virieux
 */
public class SftFilterLatchModule extends MobileItemModule {

    private boolean locked;
    private boolean unlocked;
    private boolean inError;

    private FilterPresenceInLatchStatus presenceStatus;
    private LockStatus lockStatus;

    protected final LatchActuatorModule latchActuator;
    protected NumericSensor filterPresenceSensor;
    protected NumericSensor lockSensor;
    protected NumericSensor unlockSensor;
    private CompactIOModule filterSensorsDIO;
    /**
     * This is the name of the DIO on which are plugged the filter sensors. Used
     * in the groovy file and in initModule.
     *
     */
    private final String filterSensorsDIOName;

    //Used because we have to wait for the update from the sensors to know the state
    //of the latch
    private final Condition stateUpdated = lock.newCondition();

    /**
     * This is used when we update the latch state with the values returned by
     * the sensors.
     */
    protected volatile boolean updatingState = false;
    private long timeoutForOpening;
    private long timeoutForClosing;

    public SftFilterLatchModule(String moduleName, int aTickMillis,
            long timeoutForClosing,
            long timeoutForOpening,
            String filterSensorsDIOName,
            NumericSensor lockSensor,
            NumericSensor unlockSensor,
            NumericSensor filterPresenceSensor,
            LatchActuatorModule latchActuator) {
        super(moduleName, aTickMillis);
        this.timeoutForClosing = timeoutForClosing;
        this.timeoutForOpening = timeoutForOpening;
        this.filterSensorsDIOName = filterSensorsDIOName;
        this.lockSensor = lockSensor;
        this.unlockSensor = unlockSensor;
        this.filterPresenceSensor = filterPresenceSensor;
        this.latchActuator = latchActuator;

    }

    @ConfigChanger
    public void setTimeoutForClosing(long timeoutForClosing) {
        this.timeoutForClosing = timeoutForClosing;
    }

    @ConfigChanger
    public void setTimeoutForOpening(long timeoutForOpening) {
        this.timeoutForOpening = timeoutForOpening;
    }

    public long getTimeoutForClosing() {
        return timeoutForClosing;
    }

    public long getTimeoutForOpening() {
        return timeoutForOpening;
    }

    //for FCSUtils
    public NumericSensor getFilterPresenceSensor() {
        return filterPresenceSensor;
    }

    //for FCSUtils
    public LatchActuatorModule getLatchActuator() {
        return latchActuator;
    }

    //for FCSUtils
    public NumericSensor getLockSensor() {
        return lockSensor;
    }

    //for FCSUtils
    public NumericSensor getUnlockSensor() {
        return unlockSensor;
    }

    //for FCSUtils
    public boolean isControllerInFault() {
        return false;
    }

    /**
     * ***********************************************************************************************
     */
    /**
     * ******************** END OF SETTERS AND GETTERS  **********************************************
     */
    /**
     * ***********************************************************************************************
     */
    @Override
    public void initModule() {
        this.filterSensorsDIO = (CompactIOModule) this.getComponentByName(this.filterSensorsDIOName);
        this.lockStatus = LockStatus.UNKNOWN;
        this.presenceStatus = FilterPresenceInLatchStatus.UNKNOWN;
    }

    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, description = "Returns true if hardware is connected and ready.")
    @Override
    public boolean isHardwareReady() {
        return ((MainModule) this.getComponentByName("main")).isHardwareReady();
    }

    @Override
    public boolean isActionCompleted(MobileItemAction action) {
        if (action.equals(MobileItemAction.OPEN)) {
            return this.getLockStatus().equals(LockStatus.UNLOCKED);
        } else if (action.equals(MobileItemAction.CLOSE)) {
            return this.getLockStatus().equals(LockStatus.LOCKED);
        } else {
            throw new IllegalArgumentException("Action on latch must be OPEN or CLOSE");
        }
    }

    @Override
    public void updateStateWithSensorsToCheckIfActionIsCompleted() throws Exception {
        this.updateStateWithSensors();
    }

    @Command(level = Command.ENGINEERING1, description = "Read latch sensors and update latch state", 
            type = Command.CommandType.QUERY)
    public void updateStateWithSensors() throws Exception {
        filterSensorsDIO.updateValue();

        String filterSensorsHexaValue = filterSensorsDIO.getHexaValue();

        this.updateState(filterSensorsHexaValue);
    }

    @Override
    public void startAction(MobileItemAction action) throws BadCommandException, 
            ErrorInCommandExecutionException, FcsHardwareException {
        if (action.equals(MobileItemAction.OPEN)) {
            this.latchActuator.open();
        } else if (action.equals(MobileItemAction.CLOSE)) {
            this.latchActuator.close();
        } else {
            throw new IllegalArgumentException(getName() + "Action on latch must be OPEN or CLOSE");
        }
    }

    @Override
    public void abortAction(MobileItemAction action, long delay) 
            throws BadCommandException, ErrorInCommandExecutionException, 
            FcsHardwareException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void postAction(MobileItemAction action) throws BadCommandException, 
            ErrorInCommandExecutionException, FcsHardwareException {
        publishData();
    }

    public LockStatus getLockStatus() {

        lock.lock();
        try {
            while (updatingState) {
                try {
                    this.stateUpdated.await();
                } catch (InterruptedException ex) {
                    FCSLOG.error(name + ": getLockStatus was interrupted while waiting for update.");
                }

            }
            return lockStatus;

        } finally {
            lock.unlock();
        }

    }

    public void setLockStatus(LockStatus lockStatus) {
        this.lockStatus = lockStatus;
    }

    public FilterPresenceInLatchStatus getPresenceStatus() {

        lock.lock();
        try {
            while (updatingState) {
                try {
                    this.stateUpdated.await();
                } catch (InterruptedException ex) {
                    FCSLOG.error(name + ": getPresenceStatus was interrupted while waiting for update.");
                }

            }
            return presenceStatus;

        } finally {
            lock.unlock();
        }

    }

    public StatusDataPublishedByAutochangerLatch getStatusData() {
        StatusDataPublishedByAutochangerLatch status = 
                SftUtils.createStatusDataPublishedByLatch(this);
        return status;
    }

    /**
     * Open the latch if the carousel is holding the filter.
     *
     * @return
     * @throws BadCommandException
     * @throws ErrorInCommandExecutionException
     * @throws org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException
     * @throws Exception
     */
    @Command(level = Command.ENGINEERING3, description = "Open latch", type = Command.CommandType.ACTION)
    public String open() throws BadCommandException, ErrorInCommandExecutionException, FcsHardwareException, Exception {

        this.updateStateWithSensors();

        if (!this.isLocked()) {
            return getName() + "is already OPEN";
        }

        CarouselModule carousel = (CarouselModule) this.environment.getComponentByName("carousel");

        if (!carousel.isHoldingFilterAtStandby()) {
            throw new BadCommandException(getName() + "CANNOT OPEN LATCH at STANDBY : CAROUSEL is NOT HOLDING THE FILTER");
        }

        this.executeAction(MobileItemAction.OPEN, this.timeoutForOpening);

        this.latchActuator.powerOff();

        return getName() + " is OPEN";

    }

    /**
     * Close the latch : turn on the actuator, read the sensors and wait until
     * the latch is closed. If the latch is not closed during the delai :
     * timeoutForClosing throws an Exception.
     *
     * @return
     * @throws BadCommandException
     * @throws ErrorInCommandExecutionException
     * @throws org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException
     * @throws Exception
     */
    @Command(level = Command.ENGINEERING3, description = "Close latch", type = Command.CommandType.ACTION)
    public String close() throws BadCommandException, ErrorInCommandExecutionException, FcsHardwareException, Exception {

        this.updateStateWithSensors();

        if (this.isLocked()) {
            return getName() + "is already CLOSED";
        }

        this.executeAction(MobileItemAction.CLOSE, this.timeoutForClosing);

        this.latchActuator.powerOff();

        return getName() + " is CLOSED";

    }

    /**
     *
     * @return true if the latch is closed.
     */
    @Command(level = Command.ENGINEERING1, description = "Return true if the latch is closed", type = Command.CommandType.QUERY)
    public boolean isLocked() {
        return this.getLockStatus().equals(LockStatus.LOCKED);
    }

    public void updateState(String hexaValue) {
        lock.lock();
        try {
            updatingState = true;
            this.filterPresenceSensor.updateValue(hexaValue);
            this.lockSensor.updateValue(hexaValue);
            this.unlockSensor.updateValue(hexaValue);

            if (this.filterPresenceSensor.getDigitalValue() == 1) {
                this.presenceStatus = FilterPresenceInLatchStatus.ENGAGED;
            } else {
                this.presenceStatus = FilterPresenceInLatchStatus.NOFILTER;
            }
            locked = this.lockSensor.getDigitalValue() == 1;
            unlocked = this.unlockSensor.getDigitalValue() == 1;
            inError = locked && unlocked;

            if (inError) {
                lockStatus = LockStatus.ERROR;
            } else if (locked) {
                lockStatus = LockStatus.LOCKED;
            } else if (unlocked) {
                lockStatus = LockStatus.UNLOCKED;
            } else {
                lockStatus = LockStatus.UNKNOWN;
            }

        } finally {
            updatingState = false;
            stateUpdated.signal();
            lock.unlock();
        }

        this.publishData();

    }

    /**
     * Publish Data on status bus for trending data base and GUIs.
     *
     */
    @Override
    public void publishData() {
        StatusDataPublishedByAutochangerLatch status = this.getStatusData();
        this.getSubsystem().publishSubsystemDataOnStatusBus(new KeyValueData("autochangerLatch", status));
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.name);
        sb.append("/ lock sensor=");
        sb.append(this.lockSensor.getName());
        sb.append("/ unlock sensor=");
        sb.append(this.unlockSensor.getName());
        sb.append("/ filter presence sensor=");
        sb.append(this.filterPresenceSensor.getName());
        return sb.toString();
    }

    @Override
    public void quickStopAction(MobileItemAction action, long delay) throws BadCommandException, ErrorInCommandExecutionException, FcsHardwareException {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

}

==> org-lsst-ccs-subsystem-fcs-main-4.0.0-sources.jar: org/lsst/ccs/subsystems/fcs/singlefiltertest/SftFilterLatchModule.java <==

package org.lsst.ccs.subsystems.fcs.singlefiltertest;

import java.util.concurrent.locks.Condition;
import org.lsst.ccs.bus.data.KeyValueData;
import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.subsystems.fcs.CarouselModule;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.FilterPresenceStatus;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.LockStatus;
import org.lsst.ccs.subsystems.fcs.FcsEnumerations.MobileItemAction;
import org.lsst.ccs.subsystems.fcs.MainModule;
import org.lsst.ccs.subsystems.fcs.NumericSensor;
import org.lsst.ccs.subsystems.fcs.StatusDataPublishedByAutochangerLatch;
import org.lsst.ccs.subsystems.fcs.common.MobileItemModule;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;
import org.lsst.ccs.subsystems.fcs.errors.RejectedCommandException;

/**
 * This is a model for a latch. A latch is what holds a filter in the trucks of
 * the autochanger. There is 2 latches : one on each side of the filter. The
 * state of the latches are monitoring by digital sensors : one sensor to know
 * if a filter is in the latch or not and 2 status sensors to know if the latch
 * is EngageBrakeed or ReleaseBrakeed. If the 2 status sensors are inconsistent,
 * the latch is in error.
 * This class is used in single-filter-test.
 * For the final product the class AutochangerLatchModule is used.
 *
 * @author virieux
 */
public class SftFilterLatchModule extends MobileItemModule {

    private boolean locked;
    private boolean unlocked;
    private boolean inError;

    private FilterPresenceStatus presenceStatus;
    private LockStatus lockStatus;

    protected final LatchActuatorModule latchActuator;
    protected NumericSensor filterPresenceSensor;
    protected NumericSensor lockSensor;
    protected NumericSensor unlockSensor;
    private CompactIOModule filterSensorsDIO;
    /**
     * This is the name of the DIO on which are plugged the filter sensors. Used
     * in the groovy file and in initModule.
     *
     */
    private final String filterSensorsDIOName;
    
    @ConfigurationParameter(description="timeout in milliseconds : if closing the autochanger latches last more"
            + " than this amount of time, then the subsystem goes in ERROR.")
    private long timeoutForOpening = 10000;
    
    @ConfigurationParameter(description="timeout in milliseconds : if opening the autochanger latches last more"
            + " than this amount of time, then the subsystem goes in ERROR.")    
    private long timeoutForClosing = 10000;

    //Used because we have to wait for the update from the sensors to know the state
    //of the latch
    private final Condition stateUpdated = lock.newCondition();

    /**
     * This is used when we update the latch state with the values returned by
     * the sensors.
     */
    protected volatile boolean updatingState = false;
    


    /**
     * Build a new SftFilterLatchModule
     * @param filterSensorsDIOName
     * @param lockSensor
     * @param unlockSensor
     * @param filterPresenceSensor
     * @param latchActuator 
     */
    public SftFilterLatchModule(
            String filterSensorsDIOName,
            NumericSensor lockSensor,
            NumericSensor unlockSensor,
            NumericSensor filterPresenceSensor,
            LatchActuatorModule latchActuator) {
        super(5000);
        this.filterSensorsDIOName = filterSensorsDIOName;
        this.lockSensor = lockSensor;
        this.unlockSensor = unlockSensor;
        this.filterPresenceSensor = filterPresenceSensor;
        this.latchActuator = latchActuator;
    }



    public long getTimeoutForClosing() {
        return timeoutForClosing;
    }

    public long getTimeoutForOpening() {
        return timeoutForOpening;
    }

    //for FCSUtils
    public NumericSensor getFilterPresenceSensor() {
        return filterPresenceSensor;
    }

    //for FCSUtils
    public LatchActuatorModule getLatchActuator() {
        return latchActuator;
    }

    //for FCSUtils
    public NumericSensor getLockSensor() {
        return lockSensor;
    }

    //for FCSUtils
    public NumericSensor getUnlockSensor() {
        return unlockSensor;
    }

    //for FCSUtils
    public boolean isControllerInFault() {
        return false;
    }

    /**
     * ***********************************************************************************************
     */
    /**
     * ******************** END OF SETTERS AND GETTERS  **********************************************
     */
    /**
     * ***********************************************************************************************
     */
    @Override
    public void initModule() {
        this.filterSensorsDIO = (CompactIOModule) getComponentLookup().getComponentByName(this.filterSensorsDIOName);
        this.lockStatus = LockStatus.UNKNOWN;
        this.presenceStatus = FilterPresenceStatus.UNKNOWN;
    }
     

    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING1, 
            description = "Returns true if hardware is connected and ready.")
    @Override
    public boolean isCANDevicesReady() {
        return ((MainModule) getComponentLookup().getComponentByName("main")).isCANDevicesReady();
    }

    @Override
    public boolean isActionCompleted(MobileItemAction action) {
        if (action.equals(MobileItemAction.OPEN)) {
            return this.getLockStatus().equals(LockStatus.UNLOCKED);
        } else if (action.equals(MobileItemAction.CLOSE)) {
            return this.getLockStatus().equals(LockStatus.LOCKED);
        } else {
            throw new IllegalArgumentException("Action on latch must be OPEN or CLOSE");
        }
    }

    @Override
    public void updateStateWithSensorsToCheckIfActionIsCompleted()  {
        this.updateStateWithSensors();
    }

    /**
     * Read latch sensors and update latch state.
     * @throws FcsHardwareException 
     */
    @Command(level = Command.ENGINEERING1, description = "Read latch sensors and update latch state", 
            type = Command.CommandType.QUERY)
    public void updateStateWithSensors()  {
        filterSensorsDIO.updateValue();

        String filterSensorsHexaValue = filterSensorsDIO.getHexaValue();

        this.updateState(filterSensorsHexaValue);
    }

    @Override
    public void startAction(MobileItemAction action) throws 
            FcsHardwareException {
        if (action.equals(MobileItemAction.OPEN)) {
            this.latchActuator.open();
        } else if (action.equals(MobileItemAction.CLOSE)) {
            this.latchActuator.close();
        } else {
            throw new IllegalArgumentException(getName() + "Action on latch must be OPEN or CLOSE");
        }
    }

    @Override
    public void abortAction(MobileItemAction action, long delay)  {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void postAction(MobileItemAction action)  {
        publishData();
    }

    public LockStatus getLockStatus() {

        lock.lock();
        try {
            while (updatingState) {
                try {
                    this.stateUpdated.await();
                } catch (InterruptedException ex) {
                    FCSLOG.error(name + ": getLockStatus was interrupted while waiting for update.");
                }

            }
            return lockStatus;

        } finally {
            lock.unlock();
        }

    }

    public void setLockStatus(LockStatus lockStatus) {
        this.lockStatus = lockStatus;
    }

    /**
     * Return the filter presence status. Wait if the state is beeing updated.
     * @return 
     */
    public FilterPresenceStatus getPresenceStatus() {

        lock.lock();
        try {
            while (updatingState) {
                try {
                    this.stateUpdated.await();
                } catch (InterruptedException ex) {
                    FCSLOG.error(name + ": getPresenceStatus was interrupted while waiting for update.");
                }

            }
            return presenceStatus;

        } finally {
            lock.unlock();
        }

    }

    /**
     * Returns an object to be published on the STATUS bus by AutochangerLatch.
     * @return 
     */
    public StatusDataPublishedByAutochangerLatch getStatusData() {
        return SftUtils.createStatusDataPublishedByLatch(this);
    }

    /**
     * Open the latch if the carousel is holding the filter.
     *
     * @return
     * @throws RejectedCommandException
     * @throws org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException
     */
    @Command(level = Command.ENGINEERING3, description = "Open latch", type = Command.CommandType.ACTION)
    public String open()  {

        this.updateStateWithSensors();

        if (!this.isLocked()) {
            return getName() + "is already OPEN";
        }

        CarouselModule carousel = (CarouselModule) getComponentLookup().getComponentByName("carousel");

        if (!carousel.isHoldingFilterAtStandby()) {
            throw new RejectedCommandException(getName() + "CANNOT OPEN LATCH at STANDBY : CAROUSEL is NOT HOLDING THE FILTER");
        }

        this.executeAction(MobileItemAction.OPEN, this.timeoutForOpening);

        this.latchActuator.powerOff();

        return getName() + " is OPEN";

    }

    /**
     * Close the latch : turn on the actuator, read the sensors and wait until
     * the latch is closed. If the latch is not closed during the delai :
     * timeoutForClosing throws an Exception.
     *
     * @return
     * @throws org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException
     */
    @Command(level = Command.ENGINEERING3, description = "Close latch", type = Command.CommandType.ACTION)
    public String close()  {

        this.updateStateWithSensors();

        if (this.isLocked()) {
            return getName() + "is already CLOSED";
        }

        this.executeAction(MobileItemAction.CLOSE, this.timeoutForClosing);

        this.latchActuator.powerOff();

        return getName() + " is CLOSED";

    }

    /**
     *
     * @return true if the latch is closed.
     */
    @Command(level = Command.ENGINEERING1, description = "Return true if the latch is closed", type = Command.CommandType.QUERY)
    public boolean isLocked() {
        return this.getLockStatus().equals(LockStatus.LOCKED);
    }

    /**
     * Update Latch State.
     * @param hexaValue 
     */
    public void updateState(String hexaValue) {
        lock.lock();
        try {
            updatingState = true;
            this.filterPresenceSensor.updateValue(hexaValue);
            this.lockSensor.updateValue(hexaValue);
            this.unlockSensor.updateValue(hexaValue);

            if (this.filterPresenceSensor.getDigitalValue() == 1) {
                this.presenceStatus = FilterPresenceStatus.ENGAGED;
            } else {
                this.presenceStatus = FilterPresenceStatus.NOFILTER;
            }
            locked = this.lockSensor.getDigitalValue() == 1;
            unlocked = this.unlockSensor.getDigitalValue() == 1;
            inError = locked && unlocked;

            if (inError) {
                lockStatus = LockStatus.ERROR;
            } else if (locked) {
                lockStatus = LockStatus.LOCKED;
            } else if (unlocked) {
                lockStatus = LockStatus.UNLOCKED;
            } else {
                lockStatus = LockStatus.UNKNOWN;
            }

        } finally {
            updatingState = false;
            stateUpdated.signal();
            lock.unlock();
        }

        this.publishData();

    }

    /**
     * Publish Data on status bus for trending data base and GUIs.
     *
     */
    @Override
    public void publishData() {
        StatusDataPublishedByAutochangerLatch status = this.getStatusData();
        this.getSubsystem().publishSubsystemDataOnStatusBus(new KeyValueData("autochangerLatch", status));
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.name);
        sb.append("/ lock sensor=");
        sb.append(this.lockSensor.getName());
        sb.append("/ unlock sensor=");
        sb.append(this.unlockSensor.getName());
        sb.append("/ filter presence sensor=");
        sb.append(this.filterPresenceSensor.getName());
        return sb.toString();
    }

    @Override
    public void quickStopAction(MobileItemAction action, long delay)  {
        throw new UnsupportedOperationException("Not supported yet."); 
    }

}
